/*
Copyright (c) Xerox Corporation 1998-2002.  All rights reserved.

Use and copying of this software and preparation of derivative works based
upon this software are permitted.  Any distribution of this software or
derivative works must comply with all applicable United States export control
laws.

This software is made available AS IS, and Xerox Corporation makes no warranty
about the software, its performance or its conformity to any specification.
*/

package bean;

import java.beans.*;
import java.io.Serializable;

class Point implements Serializable {

    private PropertyChangeSupport support 
    = new PropertyChangeSupport(this);

  public void addPropertyChangeListener(PropertyChangeListener listener){
    support.addPropertyChangeListener(listener);
  }

  public void addPropertyChangeListener(String propertyName,
                                              PropertyChangeListener listener){

    support.addPropertyChangeListener(propertyName, listener);
  }

  public void removePropertyChangeListener(String propertyName,
                                                 PropertyChangeListener listener) {
    support.removePropertyChangeListener(propertyName, listener);
  }

  public void removePropertyChangeListener(PropertyChangeListener listener) {
    support.removePropertyChangeListener(listener);
  }

  public void hasListeners(String propertyName) {
    support.hasListeners(propertyName);
  }

    
    protected int x = 0;
    protected int y = 0;

    /**
     * Return the X coordinate
     */
    public int getX(){
        return x;
    }

    /**
     * Return the y coordinate
     */
    public int getY(){
        return y;
    }

    /**
     * Set the x and y coordinates
     */
    public void setRectangular(int newX, int newY){
        setX(newX);
        setY(newY);
    }

    /**
     * Set the X coordinate
     */
    public void setX(int newX) {
       int oldX = x; 
       x = newX;
       support.firePropertyChange("X",new Double(oldX),new Double(x));
    }

    /**
     * set the y coordinate
     */
    public void setY(int newY) {
        int oldY = y;
	y = newY;
        support.firePropertyChange("Y",new Double(oldY),new Double(y));
    }

    /**
     * Move the point by the specified x and y offset
     */
    public void offset(int deltaX, int deltaY){
        setRectangular(x + deltaX, y + deltaY);
    }

    /**
     * Make a string of this
     */
    public String toString(){
        return "(" + getX() + ", " + getY() + ")" ;
    }

    
}
